@@ -2,24 +2,32 @@
 #'
 #' Distinguish between droplets containing cells and ambient RNA in a droplet-based single-cell RNA sequencing experiment.
 #' 
-#' @param m A numeric matrix object - usually a \linkS4class{dgTMatrix} or \linkS4class{dgCMatrix} - 
+#' @param m A numeric matrix-like object - usually a \linkS4class{dgTMatrix} or \linkS4class{dgCMatrix} - 
 #' containing droplet data \emph{prior to any filtering or cell calling}.
 #' Columns represent barcoded droplets, rows represent genes.
+#'
+#' For \code{emptyDrops}, this may also be a \linkS4class{SummarizedExperiment} object containing such a matrix.
 #' @param lower A numeric scalar specifying the lower bound on the total UMI count, 
 #' at or below which all barcodes are assumed to correspond to empty droplets.
-#' @param by.rank An integer scalar parametrizing an alternative method for identifying assumed empty droplets - see \code{?\link{estimateAmbience}} for more details.
+#' @param by.rank An integer scalar parametrizing an alternative method for identifying assumed empty droplets - see \code{?\link{ambientProfileEmpty}} for more details.
+#' If set, this is used to redefine \code{lower} and any specified value for \code{lower} is ignored.
 #' @param niters An integer scalar specifying the number of iterations to use for the Monte Carlo p-value calculations.
 #' @param test.ambient A logical scalar indicating whether results should be returned for barcodes with totals less than or equal to \code{lower}.
 #' @param ignore A numeric scalar specifying the lower bound on the total UMI count, at or below which barcodes will be ignored (see Details for how this differs from \code{lower}).
 #' @param alpha A numeric scalar specifying the scaling parameter for the Dirichlet-multinomial sampling scheme.
-#' @param BPPARAM A BiocParallelParam object indicating whether parallelization should be used to compute p-values.
+#' @param BPPARAM A \linkS4class{BiocParallelParam} object indicating whether parallelization should be used.
 #' @param retain A numeric scalar specifying the threshold for the total UMI count above which all barcodes are assumed to contain cells.
 #' @param barcode.args Further arguments to pass to \code{\link{barcodeRanks}}.
-#' @param round Logical scalar indicating whether to check for non-integer values in \code{m} and, if present, round them for ambient profile estimation (see \code{?\link{estimateAmbience}}) and the multinomial simulations.
-#' @param ... Further arguments to pass to \code{testEmptyDrops}.
+#' @param round Logical scalar indicating whether to check for non-integer values in \code{m} and, if present, round them for ambient profile estimation (see \code{?\link{ambientProfileEmpty}}) and the multinomial simulations.
+#' @param assay.type Integer or string specifying the assay containing the count matrix.
+#' @param ... For the generic, further arguments to pass to individual methods.
+#'
+#' For the SummarizedExperiment method, further arguments to pass to the ANY method.
+#'
+#' For the ANY method, further arguments to pass to \code{testEmptyDrops}.
 #' 
 #' @section Details about \code{testEmptyDrops}:
-#' The \code{testEmptyDrops} function first obtains an estimate of the composition of the ambient pool of RNA based on the barcodes with total UMI counts less than or equal to \code{lower} (see \code{?\link{estimateAmbience}} for details).
+#' The \code{testEmptyDrops} function first obtains an estimate of the composition of the ambient pool of RNA based on the barcodes with total UMI counts less than or equal to \code{lower} (see \code{?\link{ambientProfileEmpty}} for details).
 #' This assumes that a cell-containing droplet would generally have higher total counts than empty droplets containing RNA from the ambient pool.
 #' Counts for the low-count barcodes are pooled together, and an estimate of the proportion vector for the ambient pool is calculated using \code{\link{goodTuringProportions}}.
 #' The count vector for each barcode above \code{lower} is then tested for a significant deviation from these proportions.
@@ -57,6 +65,9 @@
 #' All barcodes with total counts above \code{retain} are assigned p-values of zero \emph{during correction}, reflecting our assumption that they are true positives.
 #' This ensures that their Monte Carlo p-values do not affect the correction of other genes, and also means that they will have FDR values of zero.
 #' However, their original Monte Carlo p-values are still reported in the output, as these may be useful for diagnostic purposes.
+#'
+#' This effect also means that users will not be able to recover the reported \code{FDR} by simply running \code{\link{p.adjust}} on the reported \code{PValue}.
+#' Similarly, setting \code{test.ambient=TRUE} will also modify the p-values prior to correction, see commentary below.
 #' 
 #' In general, users should call \code{emptyDrops} rather than \code{testEmptyDrops}.
 #' The latter is a \dQuote{no frills} version that is largely intended for use within other functions.
@@ -69,24 +80,29 @@
 #' If \code{alpha=Inf}, the sampling of molecules is modelled with a multinomial distribution.
 #' 
 #' Users can check whether the model is suitable by extracting the p-values for all barcodes with \code{test.ambient=TRUE}.
-#' Under the null hypothesis, the p-values for presumed ambient barcodes (i.e., with total counts below \code{lower}) should be uniformly distributed.
+#' Under the null hypothesis, the p-values for presumed ambient barcodes (i.e., with total counts less than or equal to \code{lower}) should be uniformly distributed.
 #' Skews in the p-value distribution are indicative of an inaccuracy in the model and/or its estimates (of \code{alpha} or the ambient profile).
 #' 
 #' @section \code{NA} values in the results:
-#' We assume that barcodes with total UMI counts below \code{lower} correspond to empty droplets.
+#' We assume that barcodes with total UMI counts less than or equal to \code{lower} correspond to empty droplets.
 #' These are used to estimate the ambient expression profile against which the remaining barcodes are tested.
 #' Under this definition, these low-count barcodes cannot be cell-containing droplets and are excluded from the hypothesis testing.
+#' By removing these uninteresting tests, we obtain a modest improvement in detection power for the high-count barcodes.
 #' 
 #' However, it is still desirable for the number of rows of the output DataFrame to be the same as \code{ncol(m)}.
 #' This allows easy subsetting of \code{m} based on a logical vector constructed from the output (e.g., to retain all FDR values below a threshold).
 #' To satisfy this requirement, the rows for the excluded barcodes are filled in with \code{NA} values for all fields in the output.
 #' We suggest using \code{\link{which}} to pick barcodes below a FDR threshold, see the Examples.
 #' 
-#' If \code{test.ambient=FALSE}, non-\code{NA} statistics will be reported for all barcodes.
-#' This is occasionally useful for diagnostics to ensure that the p-values are well-calibrated for barcodes below \code{lower}.
+#' If \code{test.ambient=TRUE}, non-\code{NA} p-values will be reported for all barcodes with positive total counts, including those not greater than \code{lower}.
+#' This is occasionally useful for diagnostics to ensure that the p-values are well-calibrated for barcodes corresponding to (presumably) empty droplets.
 #' Specifically, if the null hypothesis were true, p-values for low-count barcodes should have a uniform distribution.
 #' Any strong peaks in the p-values near zero indicate that \code{emptyDrops} is not controlling the FDR correctly.
 #'
+#' Note that, when setting \code{test.ambient=TRUE} in \code{\link{emptyDrops}}, barcodes less than or equal to \code{lower} will still have \code{NA} values in \code{FDR}.
+#' Such barcodes are still explicitly ignored in the correction as these are considered to be uninteresting.
+#' For back-compatibility purposes, setting \code{test.ambient=NA} will include these barcodes in the correction.
+#'
 #' @section Non-empty droplets versus cells:
 #' Technically speaking, \code{emptyDrops} is designed to identify barcodes that correspond to non-empty droplets.
 #' This is close to but not quite the same as identifying cells,
@@ -130,7 +146,7 @@
 #' out <- emptyDrops(my.counts)
 #' out
 #' 
-#' is.cell <- out$FDR <= 0.01
+#' is.cell <- out$FDR <= 0.001
 #' sum(is.cell, na.rm=TRUE)
 #'
 #' # Subsetting the matrix to the cell-containing droplets.
@@ -156,23 +172,45 @@
 #'
 #' \code{\link{defaultDrops}}, for an implementation of the cell-calling method used by CellRanger version 2.
 #'
-#' \code{\link{estimateAmbience}}, for more details on estimation of the ambient profile.
-#'
-#' \code{\link{maximumAmbience}}, for estimating the maximum possible contribution of the ambient solution to a count profile.
+#' \code{\link{ambientProfileEmpty}}, for more details on estimation of the ambient profile.
 #' 
 #' @name emptyDrops
+NULL
+
 #' @export
-#' @importFrom BiocParallel SerialParam
+#' @rdname emptyDrops
+testEmptyDrops <- function(m, lower=100, niters=10000, test.ambient=FALSE, ignore=NULL, alpha=NULL, round=TRUE, by.rank=NULL, BPPARAM=SerialParam()) {
+    ambfun <- function(mat, totals) {
+        lower <- .get_lower(totals, lower, by.rank=by.rank)
+        astats <- .compute_ambient_stats(mat, totals, lower=lower)
+        astats$metadata <- list(lower = lower)
+        astats$keep <- !astats$ambient
+        astats
+    }
+    .test_empty_drops(m=m, ambient.FUN=ambfun, niters=niters, test.ambient=test.ambient, ignore=ignore, alpha=alpha, round=round, BPPARAM=BPPARAM) 
+}
+
+#' @importFrom BiocParallel bpstart bpstop SerialParam
 #' @importFrom S4Vectors DataFrame metadata<-
 #' @importFrom Matrix colSums
-testEmptyDrops <- function(m, lower=100, niters=10000, test.ambient=FALSE, ignore=NULL, alpha=NULL, 
-    round=TRUE, by.rank=NULL, BPPARAM=SerialParam()) 
-{
+#' @importFrom scuttle .bpNotSharedOrUp
+#' @importFrom beachmat colBlockApply
+.test_empty_drops <- function(m, ambient.FUN, niters=10000, test.ambient=FALSE, ignore=NULL, alpha=NULL, round=TRUE, BPPARAM=SerialParam()) {
+    if (.bpNotSharedOrUp(BPPARAM)) {
+        bpstart(BPPARAM)
+        on.exit(bpstop(BPPARAM))
+    }
+
+    old <- .parallelize(BPPARAM)
+    on.exit(setAutoBPPARAM(old), add=TRUE)
+
+    # NOTE: this is probably fine, as you don't have that many cell-containing
+    # droplets per sample, so the sparse matrix will generally be small.
+    m <- .realize_DA_to_memory(m, BPPARAM)
+
     m <- .rounded_to_integer(m, round)
     totals <- .intColSums(m)
-    lower <- .get_lower(totals, lower, by.rank=by.rank)
-
-    astats <- .compute_ambient_stats(m, totals, lower=lower)
+    astats <- ambient.FUN(m, totals)
     m <- astats$m
     ambient <- astats$ambient
     ambient.prop <- astats$ambient.prop
@@ -187,7 +225,7 @@ testEmptyDrops <- function(m, lower=100, niters=10000, test.ambient=FALSE, ignor
     # Removing supposed ambient cells from the matrix.
     # Also removing additional cells that don't pass some total count threshold, if required.
     if (!test.ambient) {
-        keep <- !ambient
+        keep <- astats$keep
     } else {
         keep <- totals > 0L
     }
@@ -198,7 +236,8 @@ testEmptyDrops <- function(m, lower=100, niters=10000, test.ambient=FALSE, ignor
     obs.totals <- totals[keep]
 
     # Calculating the log-multinomial probability for each cell.
-    obs.P <- .compute_multinom_prob_data(obs.m, ambient.prop, alpha=alpha)
+    obs.P <- colBlockApply(obs.m, prop=ambient.prop, alpha=alpha, BPPARAM=BPPARAM, FUN=.compute_multinom_prob_data)
+    obs.P <- unlist(obs.P, use.names=FALSE)
     rest.P <- .compute_multinom_prob_rest(obs.totals, alpha=alpha)
 
     # Computing the p-value for each observed probability.
@@ -217,7 +256,7 @@ testEmptyDrops <- function(m, lower=100, niters=10000, test.ambient=FALSE, ignor
     all.lim[keep] <- limited
 
     output <- DataFrame(Total=totals, LogProb=all.lr, PValue=all.p, Limited=all.lim, row.names=colnames(m))
-    metadata(output) <- list(lower=lower, niters=niters, ambient=ambient.prop, alpha=alpha)
+    metadata(output) <- c(astats$metadata, list(niters=niters, ambient=ambient.prop, alpha=alpha))
     output
 }
 
@@ -270,12 +309,13 @@ testEmptyDrops <- function(m, lower=100, niters=10000, test.ambient=FALSE, ignor
     list(seeds=seeds.per.core, streams=streams.per.core)
 }
 
-#' @importFrom scuttle whichNonZero
-.compute_multinom_prob_data <- function(mat, prop, alpha=Inf)
-# Efficiently calculates the data-dependent component of the log-multinomial probability.
+#' @importFrom beachmat whichNonZero
+.compute_multinom_prob_data <- function(block, prop, alpha=Inf, BPPARAM=SerialParam())
+# Efficiently calculates the data-dependent component of the log-multinomial probability
+# for a column-wise chunk of the full matrix (or, indeed, the full matrix itself).
 # Also does so for the Dirichlet-multinomial log-probability for a given 'alpha'.
 {
-    nonzero <- whichNonZero(mat)
+    nonzero <- whichNonZero(block)
     i <- nonzero$i
     j <- nonzero$j
     x <- nonzero$x
@@ -289,7 +329,7 @@ testEmptyDrops <- function(m, lower=100, niters=10000, test.ambient=FALSE, ignor
 
     # No need to defend against NA values from tapply,
     # these should not be present after removal of all-zero columns.
-    j <- factor(j, levels=seq_len(ncol(mat)))
+    j <- factor(j, levels=seq_len(ncol(block)))
     obs.P <- tapply(p.n0, INDEX=j, FUN=sum)
     as.numeric(obs.P)
 }
@@ -304,7 +344,7 @@ testEmptyDrops <- function(m, lower=100, niters=10000, test.ambient=FALSE, ignor
     }
 }
 
-#' @importFrom scuttle whichNonZero
+#' @importFrom beachmat whichNonZero
 #' @importFrom stats optimize 
 .estimate_alpha <- function(mat, prop, totals, interval=c(0.01, 10000))
 # Efficiently finds the MLE for the overdispersion parameter of a Dirichlet-multinomial distribution.
@@ -330,43 +370,55 @@ testEmptyDrops <- function(m, lower=100, niters=10000, test.ambient=FALSE, ignor
     optimize(LOGLIK, interval=interval, maximum=TRUE)$maximum
 }
 
-#' @export
-#' @rdname emptyDrops
 #' @importFrom stats p.adjust
 #' @importFrom S4Vectors metadata<- metadata
-emptyDrops <- function(m, lower=100, retain=NULL, barcode.args=list(), round=TRUE, ...) 
-# Combined function that puts these all together, always keeping cells above the knee 
-# point (they are given p-values of 0, as they are always rejected). 
-# 
-# written by Aaron Lun
-# created 7 August 2017
-{
+#' @importFrom BiocParallel SerialParam
+.empty_drops <- function(m, lower=100, retain=NULL, barcode.args=list(), round=TRUE, test.ambient=FALSE, ..., BPPARAM=SerialParam()) {
+    if (.bpNotSharedOrUp(BPPARAM)) {
+        bpstart(BPPARAM)
+        on.exit(bpstop(BPPARAM))
+    }
+
+    if (correct.ambient <- is.na(test.ambient)) {
+        test.ambient <- TRUE
+    }
+
+    m <- .realize_DA_to_memory(m, BPPARAM)
     m <- .rounded_to_integer(m, round)
-    stats <- testEmptyDrops(m, lower=lower, round=FALSE, ...)
+    stats <- testEmptyDrops(m, lower=lower, round=FALSE, test.ambient=test.ambient, ..., BPPARAM=BPPARAM)
     tmp <- stats$PValue
+
+    # Possibly redefine 'lower' based on 'by.rank=' passed to testEmptyDrops.
+    lower <- metadata(stats)$lower
     
     if (is.null(retain)) {
         br.out <- do.call(barcodeRanks, c(list(m, lower=lower), barcode.args))
         retain <- metadata(br.out)$knee
     }
+    metadata(stats)$retain <- retain
     always <- stats$Total >= retain
     tmp[always] <- 0
 
-    metadata(stats)$retain <- retain
+    if (test.ambient && !correct.ambient) {
+        discard <- (stats$Total <= lower)
+        tmp[discard] <- NA_real_
+    }
+
     stats$FDR <- p.adjust(tmp, method="BH")
-    return(stats)
+    stats
 }
 
-#' @importFrom Matrix colSums rowSums
-.rounded_to_integer <- function(m, round=TRUE) {
-    if (round) {
-        cs <- colSums(m)
-        rs <- rowSums(m)
-        if (!isTRUE(all.equal(cs, round(cs))) ||
-            !isTRUE(all.equal(rs, round(rs)))) 
-        {
-            m <- round(m)
-        }
-    }
-    m
-}
+#' @export
+#' @rdname emptyDrops
+setGeneric("emptyDrops", function(m, ...) standardGeneric("emptyDrops"))
+
+#' @export
+#' @rdname emptyDrops
+setMethod("emptyDrops", "ANY", .empty_drops)
+
+#' @export
+#' @rdname emptyDrops
+#' @importFrom SummarizedExperiment assay
+setMethod("emptyDrops", "SummarizedExperiment", function(m, ..., assay.type="counts") {
+    .empty_drops(assay(m, assay.type), ...)
+})
